#include "transform.h"

//This is a basic constructor to set up with some initial values.
BlackHoleTransform::BlackHoleTransform()
{
    theImage = 0;

    centerX = 0;
    centerY = 0;
    Active = false;
    Snap = false;
    holeRadius = 1;
    horizonRadius = 2;
}

//This constructor is far more useful, as we can set the properties of our black hole.
BlackHoleTransform::BlackHoleTransform(float nx, float ny, float cRadius, float hRadius)
{
    theImage = 0;
    Active = false;
    Snap = false;
    centerX = nx;
    centerY = ny;
    holeRadius = cRadius;
    horizonRadius = hRadius;
}

//This tells the Transform what image it currently affects.
void BlackHoleTransform::SetTransform(Pixels* Target)
{
    theImage = Target;
}

//Get our center.
float BlackHoleTransform::GetX()
{
    return centerX;
}

//Get our center.
float BlackHoleTransform::GetY()
{
    return centerY;
}

//This gives us the radius of the event horizon.
float BlackHoleTransform::GetEventHorizon()
{
    return horizonRadius;
}

//This gets the radius of the innermost hole (the point of death.)
float BlackHoleTransform::GetBlackHole()
{
    return holeRadius;
}

//This affects the pixels of the chosen image.
void BlackHoleTransform::UpdateTransform()
{
    Pixel* thePixel = theImage->GetFirstPixel();       
    float radius = horizonRadius * horizonRadius;      
    while (thePixel != 0)
    {
        float xlength = centerX - (thePixel->rx + theImage->X());         //We figure out if the vector between the center
        float ylength = centerY - (thePixel->ry + theImage->Y());         //and the pixel is within the radius of the black hole.
        float length = xlength*xlength + ylength*ylength;

        if (length <= radius)  //If the pixel is to be affected....
        {
           //We need to move thePixel along the vector length a smidgen, to give the effect of a spiral.
            thePixel->a = (int) (length/radius * 200);
            length /= 40.0;                            
            if (length < 1) length = 1;

            thePixel->rx += xlength/length;
            thePixel->ry += ylength/length;
            //Then we calculate the angle relative to the center, moving the pixel about that center.
            float dCos = cos(0.04);
            float dSin = sin(0.04);
            float ox = thePixel->rx - centerX + theImage->X();
            float oy = thePixel->ry - centerY + theImage->Y();

            thePixel->rx = (ox * dCos) - (oy * dSin) + centerX - theImage->X();
            thePixel->ry = (ox * dSin) + (oy * dCos) + centerY - theImage->Y();
        }
        else  
        //If the pixel is beyond the black hole, it should still be affected.  
        //This gives the effect of 'slowly dragging' the pixel towards it's doom.
        {
            length /= 3.0;
            if (length < 1) length = 1;
            thePixel->rx += xlength/length;
            thePixel->ry += ylength/length;
        }
        thePixel = theImage->GetNextPixel();
    }
}

void BlackHoleTransform::SwitchTransform()
{
    if (Active)  //When we turn off the Transform, we'll just reset it all back to normal.
    {
        Pixel* thePixel = theImage->GetFirstPixel();
        while (thePixel != 0)
        {
            thePixel->rx = thePixel->ox;
            thePixel->ry = thePixel->oy;
            thePixel->a = thePixel->oa;
            thePixel = theImage->GetNextPixel();
        }
    }
    Active = !Active;
}

bool BlackHoleTransform::IsTransformOn()
{
    return Active;
}
