#ifndef gf_included_pixels_h
#define gf_included_pixels_h

/*
A pixel is of unit-length on the screen.  A Pixels is a matrix of pixels, mean to represent an object.
That object will resemble a sprite, unless certain mathematical models are applied.  This can result in
a twisting of the object, have certain pixels destroyed (and then created as a free pixel), or having
a sinusoidal or jagged motion added as a random element when the pixels are moved.  So when a ship moves to the
left, for example, perhaps the pixels on the right will drag behind a little bit.  When the ship is caught in
a gravity pull, perhaps it will be reduced to the centre.  Even weapons are a collection of pixels.

For the purposes of transparencies, an alpha component is also present.
*/

typedef struct pixel
{
    int r;  //These are stored in RGB color format, where 0 is black and 255 is white.
    int g;
    int b;  //The location of a pixel is stored in the matrix of the Pixels and referenced by the game from there.
    int a;
    int hit;  //A 0 or 1, determining if this pixel has hit-detection.
    float z;   //The higher this is, the later it is drawn.  This is for priority painting for pixels that come on top.

    float ox;
    float oy;  //These are the origin co-ordinates when the pixel is first defined.
    float oz;

    int oa;  //Original alpha value.
    float transformx;  //This is a reference point used by transforms.
    float transformy;

    float rx;
    float ry;  //These are motions and distances relative to the whole.
    float vx;
    float vy;
    float ax;
    float ay;
    struct pixel* next;
    struct pixel* prev;
} Pixel;

//We need a list of pixels to define an object.  These pixels should be kept organized by their z.



//This is a collection of pixels, used to define an object or an image or a background.
class Pixels
{
private:
    float originX;  //This is the origin from which all pixels are relative in position.
    float originY;

    float endX; //This is the co-ordinate for the furthest diagonal (bottom-right) pixel.  This is used for calculations.
    float endY;  //It is not relative to the screen, but relative to the origin point.

    Pixel* PixelHead;  //A dynamically created list to store each pixel.    This is the start of the list.
    Pixel* PixelCurrent; //This is where we currently are in the list of pixels.
    Pixel* PixelTail;  // The end of the list.  We use this to quickly add new pixels.
    Pixel* LastHit;   //This is the last pixel to be hit by a transform.

public:
    Pixels();
    ~Pixels();

    float X();
    float Y();
    float EndX();
    float EndY();
    void MoveEnd(float nx, float ny);   //This sets the end co-ordinates to somewhere else, in case something happened to the image.
    void SetX(float nx);
    void SetY(float ny);

    void AddPixel(float rx, float ry, int r, int g, int b, int a, int hit);
    bool CheckHit(float coord_x, float coord_y);  //This takes game co-ordinates and sees if the Pixels gets hit.
    Pixel* GetLastHit();  //This returns a pixel pointer to the last hit pixel from a previous collision.

    Pixel* GetFirstPixel();
    Pixel* GetNextPixel();
    Pixel* GetLastPixel();
};


#endif
