#include "transform.h"

void StickyTransform::SetTransform(Pixels* Target, float startX, float startY, Pixel* thePixel)
{
    theImage = Target;
    originX = startX;
    endX = originX;
    originY = startY;
    endY = originY;
    HitPixel = thePixel;
    Active = true;
    Snap = false;
}

StickyTransform::StickyTransform()
{
    theImage = 0;
    originX = 0;
    originY = 0;
    endX = 0;
    endY = 0;
    HitPixel = 0;
    Active = false;
}

void StickyTransform::SwitchTransform()
{
    if (Active)
    {
        SnapTransform();
    }
    else Active = true;
}

bool StickyTransform::IsTransformOn()
{
    return Active;
}

void StickyTransform::SnapTransform()
{
    Snap = true;
    Tolerance = 8.0;
}

void StickyTransform::UpdateTransform(float newX, float newY)
{
    if (Snap)
    {
        SnapIt();
    }
    else
    {
        StretchIt(newX, newY);
    }
}

void StickyTransform::SnapIt()
{
    Pixel* thePixel = theImage->GetFirstPixel();
    int checkSnap = 0;
    while (thePixel != 0)
    {
        if (thePixel->vx > 0)
        {
            checkSnap++;
            /*Clearly we should move rx towards transformx.  If they are the same, we set transformx to the opposite
              vector direction with a smaller length. */
            if (thePixel->rx == thePixel->transformx && thePixel->ry == thePixel->transformy)
            {
                if (thePixel->transformx > (thePixel->ox - thePixel->vx) && thePixel->transformx < (thePixel->ox + thePixel->vx))
                    //We're close enough to the origin just to snap to the origin.
                    thePixel->transformx = thePixel->ox;
                else
                {
                    thePixel->transformx -= thePixel->ox;
                    thePixel->transformx *= 0.6;
                    thePixel->transformx = thePixel->ox - thePixel->transformx;
                }
                if (thePixel->transformy > (thePixel->oy -1) && thePixel->transformy < (thePixel->oy + 1))
                    thePixel->transformy = thePixel->oy;
                else
                {
                    thePixel->transformy -= thePixel->oy;
                    thePixel->transformy *= 0.6;
                    thePixel->transformy = thePixel->oy - thePixel->transformy;
                }
            }

            float movex = (thePixel->rx - thePixel->transformx);
            float movey = (thePixel->ry - thePixel->transformy);

            if (movex > -Tolerance && movex < Tolerance) thePixel->rx = thePixel->transformx;
            if (movey > -Tolerance && movey < Tolerance) thePixel->ry = thePixel->transformy;

            if (thePixel->rx != thePixel->transformx)
            {
                thePixel->rx -= movex/4.0;
            }
            if (thePixel->ry != thePixel->transformy)
            {
                thePixel->ry -= movey/4.0;
            }

            if (thePixel->rx == thePixel->ox && thePixel->rx == thePixel->transformx &&
                    thePixel->ry == thePixel->oy && thePixel->ry == thePixel->transformy)
            {
                thePixel->vx = 0;
                thePixel->z = thePixel->oz;
            }
        }
        thePixel = theImage->GetNextPixel();
    }

    if (!checkSnap)
    {
        Active = false;
        Snap = false;
    }
    Tolerance += 2;
}

void StickyTransform::StretchIt(float newX, float newY)
{
    endX = newX;
    endY = newY;
    originX = HitPixel->ox + theImage->X();
    originY = HitPixel->oy + theImage->Y();
    float xradius = endX-originX;
    float yradius = endY-originY;
    float radius = xradius*xradius + yradius*yradius;

    if (HitPixel != 0)
    {
        //Now we delve into the image and update all affected pixels accordingly.
        //We have to take into account the position of the Image, since the pixels are set relative to that.
        //We check each pixel in our image.  If their x,y is within the circle of influence, we move it according to it's
        //distance from the centre.  The distance from the centre is calculated as a vector (leaving the length squared.)
        //If the length squared is less than the radius squared, we are within the circle.
        //The length of the radius is determined by both endpoints.

        Pixel* thePixel = theImage->GetFirstPixel();

        while (thePixel != 0)
        {
            if (thePixel == HitPixel)
            {
                HitPixel->rx = endX - theImage->X();
                HitPixel->ry = endY - theImage->Y();
                HitPixel->vx = 1;
                HitPixel->transformx = HitPixel->rx;
                HitPixel->transformy = HitPixel->ry;
            }
            else
            {
                float xlength = (thePixel->ox + theImage->X() - originX);
                float ylength = (thePixel->oy + theImage->Y() - originY);
                float length = xlength*xlength + ylength*ylength;

                if (length < radius)
                {
                    length /= 30.0;
                    if (length < 1) length = 1.1;
                    //We move the pixel towards the endpoint a certain percentage, depending on it's distance from the radius origin.
                    float movex = (endX - (thePixel->ox + theImage->X()));
                    float movey = (endY - (thePixel->oy + theImage->Y()));

                    thePixel->rx = thePixel->ox + movex/(length);
                    thePixel->ry = thePixel->oy + movey/(length);
                    thePixel->transformx = thePixel->rx;
                    thePixel->transformy = thePixel->ry;
                    thePixel->vx = length;  //This just lets me know it's gonna snap.
                    thePixel->z = length;
                }
            }

            thePixel = theImage->GetNextPixel();
        }
    }
}
