#ifndef gf_included_transform_h
#define gf_included_transform_h

#include "pixels.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>

/*
These are mathematic transforms.  The transform will hit pixels on the screen.  Those pixels
will follow the transform relative to position.  Those pixels will have their co-ordinates set
to follow the transform.
*/

/*
RUMBLE

Starting from the centre of the image, the pixels slowly explode outwards.  As they reach the outside
of the circle (defined by the length to the corner from the centre), they begin to lose their alpha value.

When the Rumble is let go, they come back together in the same manner, but much more quickly.

*/

class Transform
{
protected:
    Pixels* theImage;  //This is the sprite (or pixels) affected.
    bool Active;
    bool Snap;  //When the point is let go, the transform snaps back to it's original position.

public:
    virtual bool IsTransformOn() { }
    virtual void SwitchTransform() { }
};


/*

SHOWER

This explodes the image outwards into a shower of particles.  These particles bounce about off the walls until settling down
at the bottom of the screen.

*/

class ShowerTransform : public Transform
{
private:
    float startX;
    float startY;
    float endX;
    float endY;

    void ShowerIt();
    void SnapIt();
public:
    ShowerTransform();
    void SetTransform(Pixels* Target, float sx, float sy, float ex, float ey);
    void UpdateTransform();
    void SwitchTransform();
    bool IsTransformOn();
};

/*

TWISTY

This twists the image around until it is reversed.

*/

class TwistyTransform : public Transform
{
private:
    float moveY;   //This lets me know what part of the image is moving.
    float moveX;   //Basically, this draws a line along the given point.  Everything above the line is being twisted.
    bool Horizontal;
public:
    TwistyTransform();
    void SetTransform(Pixels* Target);
    void UpdateTransform();
    void SwitchTransform();
    bool IsTransformOn();
    void SetHorizontal();
};

/*

SIZE

This transform spreads the pixels to shrink or 'grow' an image.
*/

class SizeTransform : public Transform
{
private:
    float hMultiplier;
    float vMultiplier;
    void RedoSize();

public:
    SizeTransform();
    void SetTransform(Pixels* Target);
    void UpdateTransform();
    void SwitchTransform();
    bool IsTransformOn();

    void UpSize();
    void DownSize();
    void UpVSize();
    void UpHSize();
    void DownVSize();
    void DownHSize();
};


/*

ROTATE

This transform rotates the image about the center.  When it stops, the change is permanent.
*/

class RotateTransform : public Transform
{
private:
    float rotation;
    float centerX;
    float centerY;

public:
    RotateTransform();
    void SetTransform(Pixels* Target);
    void UpdateTransform();
    void SwitchTransform();
    bool IsTransformOn();
};

/*
BLACK HOLE

This creates a circle of a size radius with an event horizon of a further size of event radius.  If a pixel enters the circle,
it is destroyed.  If the pixel enters the event horizon, it slowly fades away as it gets closer to the circle.  It approaches
the circle in a degrading spiral (with some regards to the ongoing movement of the pixels.)

(Yes, I know that the event horizon is a term technically meant for the inner circle.  Deal with it.)

*/

class BlackHoleTransform : public Transform
{
private:
    float centerX;
    float centerY;
    float holeRadius;
    float horizonRadius;

public:
    BlackHoleTransform();
    BlackHoleTransform(float nx, float ny, float cRadius, float hRadius);
    void SetTransform(Pixels* Target);
    void UpdateTransform();
    void SwitchTransform();
    bool IsTransformOn();

    float BlackHoleTransform::GetX();
    float BlackHoleTransform::GetY();
    float BlackHoleTransform::GetEventHorizon();
    float BlackHoleTransform::GetBlackHole();
};

/*
WAVY

This alters the x-position of a particle, based on it's y-position.  It makes for a sinusoidal effect.
This transform will take the size of the affected area and render it as a rectangle.  It accounts for
the entire screen when used.

In this prototype, amplitude and frequency can be altered.

*/

class WavyTransform : public Transform
{
private:
    float startX;
    float startY;  //This is the affected rectangle.
    float endX;
    float endY;

    float tick;  //This ticks to 90 in small increments (~0.1) and then resets to 0.  This is to move the wave itself.
    bool Horizontal;  //Whether to wave it horizontally or vertically.
    float amplitude;  //This makes it wavier.
    float frequency;  //The size of the wave.
    float speed;  //This warps the image faster.

public:
    WavyTransform();
    void SetTransform(Pixels* Target, float sx, float sy, float ex, float ey);
    void UpdateTransform();
    void SwitchTransform();
    bool IsTransformOn();
    void SetAmplitude(float newAmplitude);
    void SetFrequency(float newFrequency);
    void SetSpeed(float newSpeed);
    void SetHorizontal();
};

class RumbleTransform : public Transform
{
private:
    float centerX;
    float centerY;
    float radius;

    void RumbleIt();
    void SnapIt();

public:
    RumbleTransform();
    void SetTransform(Pixels* Target);
    void UpdateTransform();
    void SwitchTransform();
    bool IsTransformOn();
};


/*
STICKY PULL
When the mouse cursor clicks, it will 'grab' onto a pixel.  Drawing from that origin will create
an invisible line towards the new cursor position.  The grabbed pixel will float to the new cursor
position.  All surrounding pixels will be drawn logarithmically towards the grabbed pixel.

This gives the appearance of 'pinching' an image and stretching it outwards.  When the grabbing
is done, the transform snaps back in the opposite direction, reverberating around the origin at
80% strength upon each snap until it is still.
*/

class StickyTransform : public Transform
{
private:
    Pixel* HitPixel;  //This is the pixel affected by this particular transform.
    float originX;
    float originY;  //This is the origin of the line.
    float endX;
    float endY;  //This is the end-point of the line and the current co-ords of the grabbed pixel.
    float Tolerance;  //This is the tolerance for motion.  It increases as the snapback continues.

    void StretchIt(float newX, float newY);  //This stretches it out towards the given point.
    void SnapIt();    //This snaps it back like an elastic band.
    void SnapTransform();

public:
    StickyTransform();
    void SetTransform(Pixels* Target, float startX, float startY, Pixel* thePixel);
    void UpdateTransform(float newX, float newY);

    bool IsTransformOn();   // Returns true if the transform is currently active.
    void SwitchTransform();  // This function turns the transform on or off.

};


#endif
